//--------------------------------------------------------------------------------------
// This file contains the basic shader for objects.
// This shader provides a standard lighting model which most objects use.
// It includes:
//	1) dot product diffuse lighting in the direction (and color) of the sun.
//	2) Ambient lighting
//--------------------------------------------------------------------------------------

//Include common values
#include "globalinclude.fxh"

float4   g_fTime;                   // App's time in seconds
float4	 g_pt3CameraPos;			// The camera position in world space

float4x4 g_mWorld;					// World Matrix
float4x4 g_mView;					// View Matrix
float4x4 g_mProj;					// Projection Matrix

float4x4 g_mWorldViewProjection;    // WVP matrix
float4x4 g_mInvWorldViewProjection;	// Inverse WVP
float4x4 g_mInvTransWorld;			// Inverse Transpose world matrix for normal transforming

//Parameters
float4 g_v3SunLightDir;		//The light direction of the sun.
float4 g_v3SunDiffuseColor;	//Diffuse lighting from the sun (color with intensity in the alpha channel).
float4 g_v3AmbientColor;	//Ambient lighting (color with intensity in the alpha channel).

//The dynamic light positions in world space
float4 g_v4LightPos[MAX_DYNAMIC_LIGHTS];
float4 g_v4LightColor;	//RGB color with intensity stored in A.

float4 g_v3ShipColor;		// The color to display the enemy ships
float4 g_spriteInfo;        // Color and size info for sprites

/*
Diffuse texture
*/
texture g_Diffuse;
sampler DiffuseSampler : register(s0) = sampler_state
{ 
    Texture = (g_Diffuse);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

/*
Sprite texture
*/
texture g_texture;
sampler SpriteSampler : register(s1) = sampler_state
{ 
    Texture = (g_texture);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

/*
Terrain atlas
*/
texture g_terrain1;
sampler TerrainSampler1 : register(s2) = sampler_state
{ 
    Texture = (g_terrain1);
    AddressU = MIRROR;
    AddressV = MIRROR;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture g_terrain2;
sampler TerrainSampler2 : register(s3) = sampler_state
{ 
    Texture = (g_terrain2);
    AddressU = MIRROR;
    AddressV = MIRROR;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture g_terrain3;
sampler TerrainSampler3 : register(s4) = sampler_state
{ 
    Texture = (g_terrain3);
    AddressU = MIRROR;
    AddressV = MIRROR;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture g_terrain4;
sampler TerrainSampler4 : register(s5) = sampler_state
{ 
    Texture = (g_terrain4);
    AddressU = MIRROR;
    AddressV = MIRROR;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture g_terrain5;
sampler TerrainSampler5 : register(s6) = sampler_state
{ 
    Texture = (g_terrain5);
    AddressU = MIRROR;
    AddressV = MIRROR;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


texture g_terrain6;
sampler TerrainSampler6 : register(s7) = sampler_state
{ 
    Texture = (g_terrain6);
    AddressU = MIRROR;
    AddressV = MIRROR;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

/*
Simple structure to provide position and normal in world space.
*/
struct BASIC_VERT1
{
    float4 Position		: POSITION;		//Position
    float3 Tangent		: TANGENT;
    float3 Binormal		: BINORMAL;
    float3 Normal		: NORMAL;	//Normal in world space
    float3 Color        : COLOR;
    float2 TexCoord		: TEXCOORD0;
    float3 WorldPos		: TEXCOORD1;	//Position in world space
};

struct BLEND_VERT
{
    float4 Position		: POSITION;		//Position
    float3 Normal		: NORMAL;	    //Normal in world space
    float4 TexCoord		: TEXCOORD0;
    float4 TexCoord1	: TEXCOORD1;	//Position in world space
    float3 WorldPos		: TEXCOORD2;	//Position in world space
};

struct VSOut
{
    float4 Position : POSITION0;
    float PSize : PSIZE0;
    float3 Color : COLOR;
};

struct PSIn
{
     float2 TexCoord : TEXCOORD0;
     float4 Color : COLOR0;
};

float3 EvalDynamicLights(float3 n, float3 worldPos )
{
	float3 retVal = float3(0,0,0);
	
	//Add in the lighting from the dynamic lights
	for( int i = 0; i < MAX_DYNAMIC_LIGHTS; ++i)
	{
		float3 l = g_v4LightPos[i].xyz - worldPos;
		float lenSqr = pow( length(l), 1.5);
		l = normalize(l);
		
		float3 h = g_pt3CameraPos.xyz + g_v4LightPos[i].xyz;
		h = h / length(h);
		
		float3 components = lit( dot( n, l), dot(n, h), 20 );
		float3 diffuse =	g_v4LightColor.xyz		//Color of the light
						*	g_v4LightColor.w			//Brightness factor
						*	components.y / lenSqr;
						
		float3 specular =	g_v4LightColor.xyz		//Color of the light
						*	g_v4LightColor.w			//Brightness factor
						*	components.z / lenSqr;
		
		retVal += g_v4LightPos[i].w				//Switch to activate/deactivate each light
					* (diffuse + specular);
	}
	
	return retVal;
}

//--------------------------------------------------------------------------------------
// Vertex shaders
//--------------------------------------------------------------------------------------
BASIC_VERT1 BasicVert1VS( BASIC_VERT1 input )
{
    BASIC_VERT1 Output = input;
    float3 vNormalWorldSpace;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(input.Position, g_mWorldViewProjection);
    
    //Push the normal forward into world space
    Output.Normal = mul(input.Normal, g_mInvTransWorld);
    
    //Pass through the worldspace position
    Output.WorldPos = mul(input.Position, g_mWorld);
    
    return Output;
}

VSOut SpriteVS( float4 Position : POSITION )
{
    VSOut Output;
    
    // figure out images to use
    
    // get angle
    //float3 worldPos = mul(Position, g_mWorld);
    //float3 camera = g_pt3CameraPos - worldPos;
    //float angle = atan(camera.y/camera.z);
    //float maxtheta = 1.0471975511965977461542144610932;
    //angle = clamp( angle, 1.221730476396030703846583537942, 1.9198621771937625346160598453375 );
    
    // Set shift value in texture
    //float index = (angle + maxtheta) / (2 * maxtheta);
    float index = 0.5;
    //index = (0.0625 * floor((1 - index) * 32));
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(Position, g_mWorldViewProjection);
    Output.PSize = g_spriteInfo.w * (1.0 + .01 * Position.z);
    Output.Color = float3(index,index,index);
    
    return Output;
}

BLEND_VERT BlendVertVS( BLEND_VERT input )
{
    BLEND_VERT Output = input;
    float3 vNormalWorldSpace;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(input.Position, g_mWorldViewProjection);
    
    //Push the normal forward into world space
    Output.Normal = mul(input.Normal, g_mInvTransWorld);
    
    //Pass through the worldspace position
    Output.WorldPos = mul(input.Position, g_mWorld);
    
    return Output;
}

//--------------------------------------------------------------------------------------
// Pixel shaders
//--------------------------------------------------------------------------------------

float4 SpritePS( PSIn In ) : COLOR0
{
    //dot(g_pt3CameraPos - In.Position, ;
    In.TexCoord.x = (In.TexCoord.x / 32) + In.Color.b;
	return tex2D(SpriteSampler,  In.TexCoord);
}

float4 RenderSunPS( BASIC_VERT1 In ) :COLOR0
{	
	float3 normal = normalize( In.Normal );
	
	//Diffuse lighting component
	float3 Diffuse = g_v3SunDiffuseColor.xyz
					* g_v3SunDiffuseColor.w
					* saturate(dot(normalize(g_v3SunLightDir), normal));
    
    //Ambient light component is not fancy
	float3 Ambient = g_v3AmbientColor.xyz * g_v3AmbientColor.w;
	float3 sun = (Ambient + Diffuse);

	//Add in the lighting from the dynamic lights
	//float3 dynamic = EvalDynamicLights(normal, In.WorldPos );
	
	//Final blend converts from linear space into gamma
    return float4( sqrt(/*dynamic +*/ sun), 1);
}

float4 RenderSunColorPS( BASIC_VERT1 In ) :COLOR0
{	
	float3 normal = normalize( In.Normal );
	
	//Diffuse lighting component
	float3 Diffuse = g_v3SunDiffuseColor.xyz
					* g_v3SunDiffuseColor.w
					* saturate(dot(normalize(g_v3SunLightDir), normal));
    
    //Ambient light component is not fancy
	float3 Ambient = g_v3AmbientColor.xyz * g_v3AmbientColor.w;
	float3 sun = (Ambient + Diffuse);

	//Add in the lighting from the dynamic lights
	//float3 dynamic = EvalDynamicLights(normal, In.WorldPos );
	
	//Final blend converts from linear space into gamma
    return float4( In.Color * sqrt(/*dynamic +*/ sun), 1);
}

float4 RenderSunAtlasPS( BLEND_VERT In ) :COLOR0
{	
    //float2 btc = float2(In.TexCoord.x / 3.0, In.TexCoord.y / 2.0);
    //float2 tc0 = btc;
    //float2 tc1 = btc;
    //tc1.x += 0.33333;
    //float2 tc2 = btc;
    //tc2.x += 0.66666;
    
    //float2 tc3 = btc;
    //tc3.y += 0.5;
    //float2 tc4 = btc;
    //tc4.y += 0.5;
    //tc4.x += 0.333;
    //float2 tc5 = btc;
    //tc5.y += 0.5;
    //tc5.x += 0.666;
    
    // determine color from blending textures;
    float3 tex0 = tex2D( TerrainSampler1, In.TexCoord.xy).xyz;
    float3 tex1 = tex2D( TerrainSampler2, In.TexCoord.xy).xyz;
    float3 tex2 = tex2D( TerrainSampler3, In.TexCoord.xy).xyz;
    float3 tex3 = tex2D( TerrainSampler4, In.TexCoord.xy).xyz;
    float3 tex4 = tex2D( TerrainSampler5, In.TexCoord.xy).xyz;
    float3 tex5 = tex2D( TerrainSampler6, In.TexCoord.xy).xyz;
    
    float3 color = (tex0 * In.TexCoord.z) + 
            (tex1 * In.TexCoord.w) + 
            (tex2 * In.TexCoord1.x) + 
            (tex3 * In.TexCoord1.y) + 
            (tex4 * In.TexCoord1.z) + 
            (tex5 * In.TexCoord1.w);

	float3 normal = normalize( In.Normal );
	
	//Diffuse lighting component
	float3 Diffuse = g_v3SunDiffuseColor.xyz
					* g_v3SunDiffuseColor.w
					* saturate(dot(normalize(g_v3SunLightDir), normal));
    
    //Ambient light component is not fancy
	float3 Ambient = g_v3AmbientColor.xyz * g_v3AmbientColor.w;
	float3 sun = (Ambient + Diffuse);

	//Add in the lighting from the dynamic lights
	//float3 dynamic = EvalDynamicLights(normal, In.WorldPos );
	
	float2 temp;
    temp = modf(In.TexCoord.xy, temp);
	
	//Final blend converts from linear space into gamma
    return float4( color * sqrt(/*dynamic +*/ sun), 1);
}


float4 RenderSunTexPS( BASIC_VERT1 In ) :COLOR0
{	
	float3 normal = normalize( In.Normal );

	//Diffuse lighting component
	float3 Tex = tex2D(DiffuseSampler, In.TexCoord.xy).xyz;
	float3 Diffuse = g_v3SunDiffuseColor.xyz
					* g_v3SunDiffuseColor.w
					* saturate(dot(normalize(g_v3SunLightDir), normalize(In.Normal)));
    
    //Ambient light component is not fancy
	float3 Ambient = g_v3AmbientColor.xyz * g_v3AmbientColor.w;
	float3 sun = (Ambient + Diffuse);

	//Add in the lighting from the dynamic lights
	float3 dynamic = EvalDynamicLights(normal, In.WorldPos );
	
	//Final blend converts from linear space into gamma
    return float4( sqrt(Tex * (dynamic + sun)), 1);
}

float4 RenderEnemyPS( BASIC_VERT1 In ):COLOR0
{
	float3 normal = normalize( In.Normal );

	//Diffuse lighting component
	float4 Tex = tex2D(DiffuseSampler, In.TexCoord.xy);
	float3 Diffuse = g_v3SunDiffuseColor.xyz
					* g_v3SunDiffuseColor.w
					* saturate(dot(normalize(g_v3SunLightDir), normalize(In.Normal)));
    
    //Ambient light component is not fancy
	float3 Ambient = g_v3AmbientColor.xyz * g_v3AmbientColor.w;
	float3 sun = (Ambient + Diffuse);

	//Add in the lighting from the dynamic lights
	float3 dynamic = EvalDynamicLights(normal, In.WorldPos );
	
	//Set the color based on the mask
	float3 texColor = Tex.xyz*Tex.w + (1 - Tex.w)*g_v3ShipColor.xyz;
	
	//Final blend converts from linear space into gamma
    return float4( sqrt(texColor * (dynamic + sun)), 1);
}

//--------------------------------------------------------------------------------------
// Technique to render an object with diffuse lighting from the sun and an ambient term
//--------------------------------------------------------------------------------------

technique RenderSun
{
    pass P0
    {          
        VertexShader = compile vs_3_0 BasicVert1VS();
        PixelShader  = compile ps_3_0 RenderSunPS();
    }
}
technique RenderSunTex
{
	pass P0
	{
        VertexShader = compile vs_3_0 BasicVert1VS();
        PixelShader  = compile ps_3_0 RenderSunTexPS();
    }
}

technique RenderSunColor
{
	pass P0
	{
		VertexShader = compile vs_3_0 BasicVert1VS();
		PixelShader = compile ps_3_0 RenderSunColorPS();
	}
}

technique RenderSunAtlas
{
	pass P0
	{
		VertexShader = compile vs_3_0 BlendVertVS();
		PixelShader = compile ps_3_0 RenderSunAtlasPS();
	}
}

technique RenderEnemy
{
	Pass P0
	{
		VertexShader = compile vs_3_0 BasicVert1VS();
		PixelShader = compile ps_3_0 RenderEnemyPS();
	}
}

technique RenderTexSprites
{
	Pass P0
	{
		VertexShader = compile vs_2_0 SpriteVS();
		PixelShader = compile ps_2_0 SpritePS();
	}
}