//--------------------------------------------------------------------------------------
// This file contains the basic shader for objects.
// This shader provides a standard lighting model which most objects use.
// It includes:
//	1) dot product diffuse lighting in the direction (and color) of the sun.
//	2) Ambient lighting
//--------------------------------------------------------------------------------------

//Include common values
#include "globalinclude.fxh"

float4   g_fTime;                   // App's time in seconds
float4x4 g_mWorld;					// World Matrix
float4x4 g_mWorldViewProjection;    // WVP matrix
float4	 g_pt3CameraPos;			// The camera position in world space

float4 g_v3SunLightDir;		//The light direction of the sun.
float4 g_v3SunDiffuseColor;	//Diffuse lighting from the sun (color with intensity in the alpha channel).

//The dynamic light positions in world space
float4 g_v4LightPos[MAX_DYNAMIC_LIGHTS];
float4 g_v4LightColor;	//RGB color with intensity stored in A.

//Cloud effects
float4 g_v4CloudTranslation;	//Translation to make the clouds appear to move.

/*
Cloud textures
*/
texture3D g_CloudNoise1;
sampler3D CloudSampler1 = sampler_state
{
	Texture = g_CloudNoise1;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

texture3D g_CloudNoise2;
sampler3D CloudSampler2 = sampler_state
{
	Texture = g_CloudNoise2;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

texture3D g_CloudNoise3;
sampler3D CloudSampler3 = sampler_state
{
	Texture = g_CloudNoise3;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

texture3D g_CloudNoise4;
sampler3D CloudSampler4 = sampler_state
{
	Texture = g_CloudNoise4;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

texture3D g_CloudNoise5;
sampler3D CloudSampler5 = sampler_state
{
	Texture = g_CloudNoise5;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

texture3D g_CloudNoise6;
sampler3D CloudSampler6 = sampler_state
{
	Texture = g_CloudNoise6;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

texture g_RenderedClouds;
sampler RenderedClouds = sampler_state
{
	Texture = g_RenderedClouds;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

/*
Simple structure to provide position and normal in world space.
*/
struct CLOUD_VERT1
{
    float4 Position		: POSITION;		//Position
    float2 TexCoord		: TEXCOORD0;
    float3 WorldPos		: TEXCOORD1;	//Position in world space
};

//--------------------------------------------------------------------------------------
// Vertex shaders
//--------------------------------------------------------------------------------------
CLOUD_VERT1 CloudScreenVS( float4 Position : POSITION,
float2 TexCoord : TEXCOORD0 )
{
    CLOUD_VERT1 Output;
    Output.TexCoord = TexCoord;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = float4(TexCoord.xy*2 - 1, .5, 1);//mul(Position, g_mWorldViewProjection);
    
    //Pass through the worldspace position
    Output.WorldPos = float3(0,0,0);//mul(Position, g_mWorld);
    
    return Output;
}

CLOUD_VERT1 CloudVertVS( float4 Position : POSITION,
float2 TexCoord : TEXCOORD0 )
{
    CLOUD_VERT1 Output;
    Output.TexCoord = TexCoord;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(Position, g_mWorldViewProjection);
    
    //Pass through the worldspace position
    Output.WorldPos = mul(Position, g_mWorld);
    
    return Output;
}

//--------------------------------------------------------------------------------------
// Pixel shaders
//--------------------------------------------------------------------------------------

//Pixel shader to render clouds
float4 RenderCloudPS( CLOUD_VERT1 In ) :COLOR0
{
	float4 speedFactors1 = float4(.005, .01, .015, .015)*g_fTime;
	float4 speedFactors2 = float4(.05, .05, 0,0)*g_fTime;
	
	float4 scaleFactors1 = float4(1.0/2.0, 1.0/4.0, 1.0/8.0, 1.0/16.0 );
	float4 scaleFactors2 = float4(1.0/32.0, 1.0/64.0, 0,0);
	
	//Add the levels of noise together
	float2 uv = In.TexCoord.xy - g_v4CloudTranslation.xy;
	float4 cloud1 = float4(0,0,0,0);
	float4 cloud2 = float4(0,0,0,0);
	cloud1.x = 2.0*tex3D(CloudSampler1, float3(uv, speedFactors1.x)).w - 1.0;
	cloud1.y = 2.0*tex3D(CloudSampler2, float3(uv, speedFactors1.y)).w - 1.0;
	cloud1.z = 2.0*tex3D(CloudSampler3, float3(uv, speedFactors1.z)).w - 1.0;
	cloud1.w = 2.0*tex3D(CloudSampler4, float3(uv, speedFactors1.w)).w - 1.0;
	cloud2.x = 2.0*tex3D(CloudSampler5, float3(uv, speedFactors2.x)).w - 1.0;
	cloud2.y = 2.0*tex3D(CloudSampler6, float3(uv, speedFactors2.y)).w - 1.0;

	float alpha = dot( scaleFactors1, cloud1 ) + dot( scaleFactors2, cloud2);
	return float4( alpha,0,0, 1);
}

float4 RenderLightingPS( CLOUD_VERT1 In ) :COLOR0
{
	float alpha = tex2D(RenderedClouds, In.TexCoord.xy).x;
	alpha = max( 1.5*alpha - .03, 0.0);
	
	float Scattering = 0.0;
	if( alpha > .00001 )
	{
		float3 EndTracePos = float3(In.TexCoord.xy, alpha);
		float3 TraceDir = normalize( -float3(1.0/20.0, 1.0/20.0, 1.0)*normalize(g_v3SunLightDir.xyz) );

		float3 currTracePos = EndTracePos - .2*TraceDir;
		TraceDir *= .08/15.0;
		for(int i = 0; i < 15; ++i)
		{
			currTracePos += TraceDir;
			float tex2 = tex2D(RenderedClouds, currTracePos.xy).x;
			Scattering += .05*step( currTracePos.z, 3*tex2);
		}
		Scattering = 1.0 / (exp(Scattering*.4));
	}

	float3 pos = In.WorldPos.xyz; pos.z += 10.0*alpha;
	float blend = 0.0;
	for( int i = 0; i < MAX_DYNAMIC_LIGHTS; ++i)
	{
		blend += 1.0 / pow( length(g_v4LightPos[i].xyz - pos ), 2.0);
	}
	blend = saturate(blend);
	
	float3 cloudColor = Scattering.xxx;
	
	alpha = 1.0 - pow( .94, alpha*255.0 );
	return float4( lerp(cloudColor, g_v4LightColor.xyz, blend), alpha);
}

//--------------------------------------------------------------------------------------
// Technique to render an object with diffuse lighting from the sun and an ambient term
//--------------------------------------------------------------------------------------
technique RenderCloud
{
	pass P0
	{
		VertexShader = compile vs_3_0 CloudScreenVS();
		PixelShader = compile ps_3_0 RenderCloudPS();
	}
}

technique RenderLighting
{
	pass P0
	{
		VertexShader = compile vs_3_0 CloudVertVS();
		PixelShader = compile ps_3_0 RenderLightingPS();
	}
}