//--------------------------------------------------------------------------------------
// This file contains the basic shader for objects.
// This shader provides a standard lighting model which most objects use.
// It includes:
//	1) dot product diffuse lighting in the direction (and color) of the sun.
//	2) Ambient lighting
//--------------------------------------------------------------------------------------

//Include common values
#include "globalinclude.fxh"

float4		g_fTime;					//App's time in seconds
float4		g_fTranslation;				//Amount to translate the map texture

/*
HUD textures
*/
texture g_HUDTex;
sampler HUDSampler = sampler_state
{
	Texture = g_HUDTex;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;
    AddressW = WRAP;
};

/*
Simple structure to provide position and normal in world space.
*/
struct HUD_VERT1
{
    float4 Position		: POSITION;		//Position
    float2 TexCoord		: TEXCOORD0;
    float3 WorldPos		: TEXCOORD1;	//Position in world space
};

//--------------------------------------------------------------------------------------
// Vertex shaders
//--------------------------------------------------------------------------------------
HUD_VERT1 HUDMoveVS( float4 Position : POSITION,
float2 TexCoord : TEXCOORD0 )
{
    HUD_VERT1 Output;
    Output.TexCoord = TexCoord;
    
    // Transform the position from object space to homogeneous projection space
    float2 transPos = 1.0*(Position.xy*2 - 1) + g_fTranslation.xy;
    Output.Position = float4(transPos, .5, 1);
    
    //Pass through the worldspace position
    Output.WorldPos = float3(0,0,0);
    
    return Output;
}

HUD_VERT1 HUDScreenVS( float4 Position : POSITION,
float2 TexCoord : TEXCOORD0 )
{
    HUD_VERT1 Output;
    Output.TexCoord = TexCoord;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = float4(1.0*(Position.xy*2 - 1), .5, 1);
    
    //Pass through the worldspace position
    Output.WorldPos = float3(0,0,0);
    
    return Output;
}

//--------------------------------------------------------------------------------------
// Pixel shaders
//--------------------------------------------------------------------------------------
float4 RenderHUDPS( HUD_VERT1 In ) :COLOR0
{
	float4 color = tex2D(HUDSampler, In.TexCoord);
	return float4(color);
}

float4 RenderBlackPS(): COLOR
{
	return float4(0,0,0,1);
}

//--------------------------------------------------------------------------------------
// Technique to render an object with diffuse lighting from the sun and an ambient term
//--------------------------------------------------------------------------------------
technique RenderHUD
{
	pass P0
	{
		VertexShader = compile vs_3_0 HUDScreenVS();
		PixelShader = compile ps_3_0 RenderHUDPS();
	}
}

technique RenderMove
{
	pass P0
	{
		VertexShader = compile vs_3_0 HUDMoveVS();
		PixelShader = compile ps_3_0 RenderHUDPS();
	}
}

technique RenderBlack
{
	pass P0
	{
		VertexShader = compile vs_3_0 HUDScreenVS();
		PixelShader = compile ps_3_0 RenderBlackPS();
	}
}