//--------------------------------------------------------------------------------------
// This file contains the shaders/effects for shading weapons fire
//--------------------------------------------------------------------------------------

//Include common values
#include "globalinclude.fxh"

float4   g_fTime;                   // App's time in seconds
float4	 g_pt3CameraPos;			// The camera position in world space

float4x4 g_mWorld;					// World Matrix
float4x4 g_mView;					// View Matrix
float4x4 g_mProj;					// Projection Matrix

float4x4 g_mWorldViewProjection;    // WVP matrix
float4x4 g_mInvWorldViewProjection;	// Inverse WVP
float4x4 g_mInvTransWorld;			// Inverse Transpose world matrix for normal transforming

//Parameters
float4 g_v3SunLightDir;		//The light direction of the sun.
float4 g_v3SunDiffuseColor;	//Diffuse lighting from the sun (color with intensity in the alpha channel).
float4 g_v3AmbientColor;	//Ambient lighting (color with intensity in the alpha channel).

//The dynamic light positions in world space
float4 g_v4LightPos[MAX_DYNAMIC_LIGHTS];
float4 g_v4LightColor;	//RGB color with intensity stored in A.

/*
Simple structure to provide position and normal in world space.
*/
struct WEAPON_VERT1
{
    float4 Position		: POSITION;		//Position
    float2 TexCoord		: TEXCOORD0;
    float3 WorldPos		: TEXCOORD1;	//Position in world space
};

//Texture used for the Laser
texture g_Laser;
sampler LaserSamper : register(s3) = sampler_state
{ 
    Texture = (g_Laser);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//Noise textrue
texture g_Noise;
sampler NoiseSampler : register(s4) = sampler_state
{
	Texture = (g_Noise);
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
//       color with diffuse material color
//--------------------------------------------------------------------------------------
WEAPON_VERT1 WeaponVS( WEAPON_VERT1 input )
{
	WEAPON_VERT1 output = input;
    float3 vNormalWorldSpace;
    
    // Transform the position from object space to homogeneous projection space
    output.Position = mul(input.Position, g_mWorldViewProjection);
    
    //Pass through the worldspace position
    output.WorldPos = mul(input.Position, g_mWorld);
    
	return output;
}

float4 WeaponPS( WEAPON_VERT1 input ) :COLOR0
{	
	float4 val = tex2D(LaserSamper, input.TexCoord.xy);
	
	float2 samplePoint = .15*( (input.TexCoord.xy) + input.WorldPos.xy);
	
	float scale = pow(1.1*input.TexCoord.y + .2, 4);
	
	float noise = 2*tex2D(NoiseSampler, samplePoint ).w - .5;
	return float4(val.xyz, val.w*(scale + noise));
}

float4 GrapplePS( WEAPON_VERT1 input ) :COLOR0
{	
	return float4(0.5, 0.5, 0.5, 1);
}

//--------------------------------------------------------------------------------------
// Technique to render an object with diffuse lighting from the sun and an ambient term
//--------------------------------------------------------------------------------------
technique RenderLaser
{
    pass P0
    {          
        VertexShader = compile vs_3_0 WeaponVS();
        PixelShader  = compile ps_3_0 WeaponPS();
    }
}

technique RenderGrapple
{
    pass P0
    {          
        VertexShader = compile vs_3_0 WeaponVS();
        PixelShader  = compile ps_3_0 GrapplePS();
    }
}
