//--------------------------------------------------------------------------------------
// This file is a heavily modified version of the file described in this header 
//  created by John Kloetzli on 9/26/2007 for CMSC691G.
//
// File: BasicHLSL.fx
//
// The effect file for the BasicHLSL sample.
//--------------------------------------------------------------------------------------

//Include common values
#include "parameters.fxh"

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float4x4 g_mWorld;                  // World matrix

float4 g_MaterialAmbientColor;      // Material's ambient color
float4 g_MaterialDiffuseColor;      // Material's diffuse color

float3 g_LightDir;					// Light's direction in world space
float4 g_LightDiffuse;				// Light's diffuse color
float4 g_LightAmbient;              // Light's ambient color

texture g_NormalMap;				
texture g_WordsTex;
texture g_MeshTexture;              // Color texture for mesh
texture g_3DNoiseTex;				// 3D noise texture for randomness

//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <g_MeshTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler Noise3DSampler = 
sampler_state
{
	Texture = <g_3DNoiseTex>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

sampler WordsSampler = 
sampler_state
{
	Texture = <g_WordsTex>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

sampler NormalSampler = 
sampler_state
{
	Texture = <g_NormalMap>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

float3 CreateStripes(float3 LocalPos)
{
	LocalPos = LocalPos + .1*( tex3D(Noise3DSampler, LocalPos*float3(1,1,1)).www -.5);

	float mask = step(LocalPos.x, 0.425)*step(-LocalPos.x, -0.275)*( step(LocalPos.y, -.75) + step(-LocalPos.y, -.75) )
			+ step(LocalPos.x, 0.075)*step(-LocalPos.x, 0.075)*( step(LocalPos.y, -.85) + step(-LocalPos.y, -.85) )
			+ step(-LocalPos.x, 0.425)*step(LocalPos.x, -0.275)*( step(LocalPos.y, -.75) + step(-LocalPos.y, -.75) )
			
			+ step(LocalPos.y, 0.425)*step(-LocalPos.y, -0.275)*( step(LocalPos.x, -.75) + step(-LocalPos.x, -.75) )
			+ step(LocalPos.y, 0.075)*step(-LocalPos.y, 0.075)*( step(LocalPos.x, -.85) + step(-LocalPos.x, -.85) )
			+ step(-LocalPos.y, 0.425)*step(LocalPos.y, -0.275)*( step(LocalPos.x, -.75) + step(-LocalPos.x, -.75) );
	float stripes = mask*float3( 1, 1, 1 );
	
	return float3(.8, .8, .8) * saturate(stripes);
}

//Sample the albedo texture
float3 SampleAlbedo(float2 texCoords)
{
	float4 texVal = tex2D( MeshTextureSampler, texCoords);
	return saturate(texVal.xyz/texVal.w);
}

//Sample the normal texture
float3 SampleNormal(float2 texCoords)
{
	float3 Normal = tex2D(NormalSampler, texCoords).xyz*2.0 - 1.0;
	return Normal;
}

//Set the lettering glowing
float3 SampleWords(float2 texCoords)
{
	float val = tex2D( WordsSampler, texCoords).w;
	val = (sin(g_fTime) + 1.0)*val;
	return saturate(float3(1.0,1.0,.2)*val);
}

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position		: POSITION;   // vertex position 
    float2 TextureUV	: TEXCOORD0;  // vertex texture coords
    float3 Normal		: TEXCOORD1;
    float3 ModelPos		: TEXCOORD2;
    float3 Binormal		: TEXCOORD3;
    float3 Tangent		: TEXCOORD4;
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos			: POSITION,
						float4 vBinormal	: BINORMAL,
						float4 vTangent		: TANGENT,
						float3 vNormal		: NORMAL,
						float2 vTexCoord0	: TEXCOORD0 )
{
    VS_OUTPUT Output;
    float3 vNormalWorldSpace;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, g_mWorldViewProjection);
    Output.TextureUV = vTexCoord0;
    
    //Push the normal forward into world space
    Output.Normal = mul(vNormal, g_mInvTransWorld);
    Output.Binormal = mul(vBinormal, g_mInvTransWorld);
    Output.Tangent = mul(vTangent, g_mInvTransWorld);
    
    Output.ModelPos = vPos;
    
    return Output;    
}


VS_OUTPUT RenderScreenVS(	float4 vPos			: POSITION,
							float4 vBinormal	: BINORMAL,
							float4 vTangent		: TANGENT,
							float3 vNormal		: NORMAL,
							float2 vTexCoord0	: TEXCOORD0 )
{
    VS_OUTPUT Output;
    float3 vNormalWorldSpace;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = float4((vTexCoord0.xy-.5)*float2(2,-2), .5, 1.0);
    Output.TextureUV = vTexCoord0;
    
    //Push the normal forward into world space
    Output.Normal = mul(vNormal, g_mInvTransWorld);
    Output.Binormal = mul(vBinormal, g_mInvTransWorld);
    Output.Tangent = mul(vTangent, g_mInvTransWorld);
    
    Output.ModelPos = vPos;
    
    return Output;    
}

//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
//       color with diffuse material color
//--------------------------------------------------------------------------------------
float4 RenderScenePS( VS_OUTPUT In ) :COLOR0
{
	//Rotate the normal from the map back into world space
	float3 Normal = mul( SampleNormal(In.TextureUV), float3x3(In.Binormal, In.Tangent, In.Normal ) );
	
	
	//Diffuse lighting component is based upon the SH coefficients
	float3 Diffuse = .5*ComputeLighting(Normal);
	
	//Albedo is mixture of texture lookups and implicit definitions
    float3 Albedo = SampleAlbedo(In.TextureUV) 
        + SampleWords(In.TextureUV)
		+ CreateStripes(In.ModelPos);
    
    //Ambient light component is not fancy
	float3 Ambient = float3(.6,.6,.6);

    return float4( Albedo*(Ambient + Diffuse), 1);
    //return float4(Normal.xyz, 1);
}

float4 RenderScreenPS( VS_OUTPUT In) :COLOR0
{
	return float4(1,1,1,1);
}

//--------------------------------------------------------------------------------------
// Renders scene to render target
//--------------------------------------------------------------------------------------
technique RenderSceneWithTexture1Light
{
    pass P0
    {          
        VertexShader = compile vs_3_0 RenderSceneVS();
        PixelShader  = compile ps_3_0 RenderScenePS();
    }
}

technique RenderScreen
{
    pass P0
    {          
        VertexShader = compile vs_3_0 RenderScreenVS();
        PixelShader  = compile ps_3_0 RenderScreenPS();
    }
}

